:- module(sq_prolog, [squares_prolog/4]).

% Square Size is covered by distinct squares of size Ss
% with coordinates XYs.
squares_prolog(Ss, Size, Xs, Ys) :-
	triples(Ss, Xs, Ys, SXYs),
	Y0 is Size+1, XY0 = 1-Y0, NSize is -Size,
	filled_hole([NSize,Size,Size], _, XY0, SXYs, []).

% triples(Ss, Xs, Ys, SXYs): SXYs is a list of s(S,X,Y)-s, where 
% S, X, and Y are corresponding elements of Ss, Xs, and Ys.
triples([S|Ss], [X|Xs], [Y|Ys], [s(S,X,Y)|SXYs]) :-
	triples(Ss, Xs, Ys, SXYs).
triples([], [], [], []).

% filled_hole(L0, L, XY, SXYs0, SXYs): Hole in line L0 starting at
% point XY, filled with squares SXYs0-SXYs (diff list) gives line L.
% Def: h(L): sum of lengths of vertical lines in L.
% Pre: All elements of L0 except the first >= 0.
% Post: All elems in L >=0, h(L0) = h(L).
filled_hole(L, L, _, SXYs, SXYs) :-
	L = [V|_], V >= 0, !.
filled_hole([V|HL], L, X0-Y0, SXYs00, SXYs) :-
	V < 0 , Y1 is Y0+V,  select(s(S,X0,Y1), SXYs00, SXYs0),
	placed_square(S, HL, L1),  Y2 is Y1+S, X2 is X0+S,
	filled_hole(L1, L2, X2-Y2, SXYs0, SXYs1),  V1 is V+S ,
	filled_hole([V1,S|L2], L, X0-Y0, SXYs1, SXYs).

% placed_square(S, HL, L): placing a square on
% HL horizontal line gives (vertical) line L.
% Pre: all elems in HL >=0
% Post: all in L except first >=0, h(L) = h(HL)-S.
placed_square(S, [H,0,H1|L], L1) :-
	S > H, !, H2 is H+H1,
	placed_square(S, [H2|L], L1).
placed_square(S, [S,V|L], [X|L]) :-
        !, X is V-S.
placed_square(S, [H|L], [X,Y|L]) :-
        S < H, X is -S, Y is H-S.

% A version of lists:select/3 with backtrack count.
select(S, [S|Ss0], Ss0).
select(S, [S0|Ss0], [S0|Ss1]) :-
%	bb_get(user:count,C), C1 is C+1, bb_put(user:count,C1),
	select(S, Ss0, Ss1).
